@@ -8,20 +8,22 @@ import torch.optim as optim
 from ai_traineree import DEVICE
 from ai_traineree.agents import AgentBase
 from ai_traineree.agents.agent_utils import soft_update
-from ai_traineree.buffers import NStepBuffer, PERBuffer
 from ai_traineree.buffers.buffer_factory import BufferFactory
+from ai_traineree.buffers.nstep import NStepBuffer
+from ai_traineree.buffers.per import PERBuffer
 from ai_traineree.loggers import DataLogger
 from ai_traineree.networks.heads import RainbowNet
-from ai_traineree.types import ActionType, AgentState, BufferState, DoneType, NetworkState, ObsType, RewardType
+from ai_traineree.types import AgentState, BufferState, NetworkState
 from ai_traineree.types.dataspace import DataSpace
+from ai_traineree.types.experience import Experience
 from ai_traineree.utils import to_numbers_seq, to_tensor
 
 
 class RainbowAgent(AgentBase):
     """Rainbow agent as described in [1].
 
-    Rainbow is a DQN agent with some improvments that were suggested before 2017.
-    As mentioned by the authors it's not exhaustive improvment but all changes are in
+    Rainbow is a DQN agent with some improvements that were suggested before 2017.
+    As mentioned by the authors it's not exhaustive improvement but all changes are in
     relatively separate areas so their connection makes sense. These improvements are:
     * Priority Experience Replay
     * Multi-step
@@ -33,7 +35,8 @@ class RainbowAgent(AgentBase):
     Consider this class as a particular version of the DQN agent.
 
     [1] "Rainbow: Combining Improvements in Deep Reinforcement Learning" by Hessel et al. (DeepMind team)
-    https://arxiv.org/abs/1710.02298
+        https://arxiv.org/abs/1710.02298
+
     """
 
     model = "Rainbow"
@@ -42,9 +45,9 @@ class RainbowAgent(AgentBase):
         self,
         obs_space: DataSpace,
         action_space: DataSpace,
-        state_transform: Optional[Callable]=None,
-        reward_transform: Optional[Callable]=None,
-        **kwargs
+        state_transform: Optional[Callable] = None,
+        reward_transform: Optional[Callable] = None,
+        **kwargs,
     ):
         """
         A wrapper over the DQN thus majority of the logic is in the DQNAgent.
@@ -58,7 +61,7 @@ class RainbowAgent(AgentBase):
             state_transform (optional func):
             reward_transform (optional func):
 
-        Keyword parameters:
+        Keyword arguments:
             pre_network_fn (function that takes input_shape and returns network):
                 Used to preprocess state before it is used in the value- and advantage-function in the dueling nets.
             hidden_layers (tuple of ints): Shape of the hidden layers in fully connected network. Default: (100, 100).
@@ -72,7 +75,7 @@ class RainbowAgent(AgentBase):
             number_updates (int): How many times to use learning step in the learning phase. Default: 1.
             max_grad_norm (float): Maximum norm of the gradient used in learning. Default: 10.
             using_double_q (bool): Whether to use Double Q Learning network. Default: True.
-            n_steps (int): Number of lookahead steps when estimating reward. See :ref:`NStepBuffer`. Default: 3.
+            n_steps (int): Number of lookahead steps when estimating reward. See :class:`NStepBuffer`. Default: 3.
             v_min (float): Lower bound for distributional value V. Default: -10.
             v_max (float): Upper bound for distributional value V. Default: 10.
             num_atoms (int): Number of atoms (discrete states) in the value V distribution. Default: 21.
@@ -83,19 +86,19 @@ class RainbowAgent(AgentBase):
 
         self.obs_space = obs_space
         self.action_space = action_space
-        self._config['obs_space'] = self.obs_space
-        self._config['action_space'] = self.action_space
+        self._config["obs_space"] = self.obs_space
+        self._config["action_space"] = self.action_space
         self.action_size = action_space.to_feature()
 
-        self.lr = float(self._register_param(kwargs, 'lr', 3e-4))
-        self.gamma = float(self._register_param(kwargs, 'gamma', 0.99))
-        self.tau = float(self._register_param(kwargs, 'tau', 0.002))
-        self.update_freq = int(self._register_param(kwargs, 'update_freq', 1))
-        self.batch_size = int(self._register_param(kwargs, 'batch_size', 80, update=True))
-        self.buffer_size = int(self._register_param(kwargs, 'buffer_size', int(1e5), update=True))
-        self.warm_up = int(self._register_param(kwargs, 'warm_up', 0))
-        self.number_updates = int(self._register_param(kwargs, 'number_updates', 1))
-        self.max_grad_norm = float(self._register_param(kwargs, 'max_grad_norm', 10))
+        self.lr = float(self._register_param(kwargs, "lr", 3e-4))
+        self.gamma = float(self._register_param(kwargs, "gamma", 0.99))
+        self.tau = float(self._register_param(kwargs, "tau", 0.002))
+        self.update_freq = int(self._register_param(kwargs, "update_freq", 1))
+        self.batch_size = int(self._register_param(kwargs, "batch_size", 80, update=True))
+        self.buffer_size = int(self._register_param(kwargs, "buffer_size", int(1e5), update=True))
+        self.warm_up = int(self._register_param(kwargs, "warm_up", 0))
+        self.number_updates = int(self._register_param(kwargs, "number_updates", 1))
+        self.max_grad_norm = float(self._register_param(kwargs, "max_grad_norm", 10))
 
         self.iteration: int = 0
         self.using_double_q = bool(self._register_param(kwargs, "using_double_q", True))
@@ -123,19 +126,19 @@ class RainbowAgent(AgentBase):
 
         self.optimizer = optim.Adam(self.net.parameters(), lr=self.lr)
         self.dist_probs = None
-        self._loss = float('nan')
+        self._loss = float("nan")
 
     @property
     def loss(self):
-        return {'loss': self._loss}
+        return {"loss": self._loss}
 
     @loss.setter
     def loss(self, value):
         if isinstance(value, dict):
-            value = value['loss']
+            value = value["loss"]
         self._loss = value
 
-    def step(self, obs: ObsType, action: ActionType, reward: RewardType, next_obs: ObsType, done: DoneType) -> None:
+    def step(self, experience: Experience) -> None:
         """Letting the agent to take a step.
 
         On some steps the agent will initiate learning step. This is dependent on
@@ -149,15 +152,19 @@ class RainbowAgent(AgentBase):
             done: (bool) Whether in terminal (end of episode) state.
 
         """
-        assert isinstance(action, int), "Rainbow expects discrete action (int)"
+        assert isinstance(experience.action, int), "Rainbow expects discrete action (int)"
         self.iteration += 1
-        t_obs = to_tensor(self.state_transform(obs)).float().to("cpu")
-        t_next_obs = to_tensor(self.state_transform(next_obs)).float().to("cpu")
-        reward = self.reward_transform(reward)
+        t_obs = to_tensor(self.state_transform(experience.obs)).float().to("cpu")
+        t_next_obs = to_tensor(self.state_transform(experience.next_obs)).float().to("cpu")
+        reward = self.reward_transform(experience.reward)
 
         # Delay adding to buffer to account for n_steps (particularly the reward)
         self.n_buffer.add(
-            state=t_obs.numpy(), action=[int(action)], reward=[reward], done=[done], next_state=t_next_obs.numpy()
+            obs=t_obs.numpy(),
+            action=[int(experience.action)],
+            reward=[reward],
+            done=[experience.done],
+            next_obs=t_next_obs.numpy(),
         )
         if not self.n_buffer.available:
             return
@@ -174,7 +181,7 @@ class RainbowAgent(AgentBase):
             # Update networks only once - sync local & target
             soft_update(self.target_net, self.net, self.tau)
 
-    def act(self, obs: ObsType, eps: float = 0.) -> int:
+    def act(self, experience: Experience, eps: float = 0.0) -> Experience:
         """
         Returns actions for given state as per current policy.
 
@@ -187,12 +194,14 @@ class RainbowAgent(AgentBase):
         if self._rng.random() < eps:
             # TODO: Update with action_space.sample() once implemented
             assert len(self.action_space.shape) == 1, "Only 1D is supported right now"
-            return self._rng.randint(self.action_space.low, self.action_space.high)
+            action = self._rng.randint(self.action_space.low, self.action_space.high)
+            return experience.update(action=action)
 
-        t_obs = to_tensor(self.state_transform(obs)).float().unsqueeze(0).to(self.device)
+        t_obs = to_tensor(self.state_transform(experience.obs)).float().unsqueeze(0).to(self.device)
         self.dist_probs = self.net.act(t_obs)
         q_values = (self.dist_probs * self.z_atoms).sum(-1)
-        return int(q_values.argmax(-1))  # Action maximizes state-action value Q(s, a)
+        action = int(q_values.argmax(-1))  # Action maximizes state-action value Q(s, a)
+        return experience.update(action=action)
 
     def learn(self, experiences: Dict[str, List]) -> None:
         """
@@ -202,30 +211,30 @@ class RainbowAgent(AgentBase):
                 Each key contains a array and all arrays have to have the same length.
 
         """
-        rewards = to_tensor(experiences['reward']).float().to(self.device)
-        dones = to_tensor(experiences['done']).type(torch.int).to(self.device)
-        states = to_tensor(experiences['state']).float().to(self.device)
-        next_states = to_tensor(experiences['next_state']).float().to(self.device)
-        actions = to_tensor(experiences['action']).type(torch.long).to(self.device)
+        rewards = to_tensor(experiences["reward"]).float().to(self.device)
+        dones = to_tensor(experiences["done"]).type(torch.int).to(self.device)
+        obss = to_tensor(experiences["obs"]).float().to(self.device)
+        next_obss = to_tensor(experiences["next_obs"]).float().to(self.device)
+        actions = to_tensor(experiences["action"]).type(torch.long).to(self.device)
         assert rewards.shape == dones.shape == (self.batch_size, 1)
-        assert states.shape == next_states.shape == (self.batch_size,) + self.obs_space.shape
+        assert obss.shape == next_obss.shape == (self.batch_size,) + self.obs_space.shape
         assert actions.shape == (self.batch_size, 1)  # Discrete domain
 
         with torch.no_grad():
-            prob_next = self.target_net.act(next_states)
+            prob_next = self.target_net.act(next_obss)
             q_next = (prob_next * self.z_atoms).sum(-1) * self.z_delta
             if self.using_double_q:
-                duel_prob_next = self.net.act(next_states)
+                duel_prob_next = self.net.act(next_obss)
                 a_next = torch.argmax((duel_prob_next * self.z_atoms).sum(-1), dim=-1)
             else:
                 a_next = torch.argmax(q_next, dim=-1)
 
             prob_next = prob_next[self.__batch_indices, a_next, :]
 
-        m = self.net.dist_projection(rewards, 1 - dones, self.gamma ** self.n_steps, prob_next)
+        m = self.net.dist_projection(rewards, 1 - dones, self.gamma**self.n_steps, prob_next)
         assert m.shape == (self.batch_size, self.num_atoms)
 
-        log_prob = self.net(states, log_prob=True)
+        log_prob = self.net(obss, log_prob=True)
         assert log_prob.shape == (self.batch_size,) + self.action_size + (self.num_atoms,)
         log_prob = log_prob[self.__batch_indices, actions.squeeze(), :]
         assert log_prob.shape == m.shape == (self.batch_size, self.num_atoms)
@@ -242,9 +251,9 @@ class RainbowAgent(AgentBase):
         self.optimizer.step()
         self._loss = float(loss.item())
 
-        if hasattr(self.buffer, 'priority_update'):
+        if hasattr(self.buffer, "priority_update"):
             assert (~torch.isnan(error)).any()
-            self.buffer.priority_update(experiences['index'], error.detach().cpu().numpy())
+            self.buffer.priority_update(experiences["index"], error.detach().cpu().numpy())
 
         # Update networks - sync local & target
         soft_update(self.target_net, self.net, self.tau)
@@ -258,7 +267,7 @@ class RainbowAgent(AgentBase):
         """
         return {"net": self.net.state_dict(), "target_net": self.target_net.state_dict()}
 
-    def log_metrics(self, data_logger: DataLogger, step: int, full_log: bool=False):
+    def log_metrics(self, data_logger: DataLogger, step: int, full_log: bool = False):
         data_logger.log_value("loss/agent", self._loss, step)
 
         if full_log and self.dist_probs is not None:
@@ -266,11 +275,17 @@ class RainbowAgent(AgentBase):
             action_size = self.action_size[0]
             for action_idx in range(action_size):
                 dist = self.dist_probs[0, action_idx]
-                data_logger.log_value(f'dist/expected_{action_idx}', (dist*self.z_atoms).sum().item(), step)
+                data_logger.log_value(f"dist/expected_{action_idx}", (dist * self.z_atoms).sum().item(), step)
                 data_logger.add_histogram(
-                    f'dist/Q_{action_idx}', min=self.z_atoms[0], max=self.z_atoms[-1], num=len(self.z_atoms),
-                    sum=dist.sum(), sum_squares=dist.pow(2).sum(), bucket_limits=self.z_atoms+self.z_delta,
-                    bucket_counts=dist, global_step=step
+                    f"dist/Q_{action_idx}",
+                    min=self.z_atoms[0],
+                    max=self.z_atoms[-1],
+                    num=len(self.z_atoms),
+                    sum=dist.sum(),
+                    sum_squares=dist.pow(2).sum(),
+                    bucket_limits=self.z_atoms + self.z_delta,
+                    bucket_counts=dist,
+                    global_step=step,
                 )
 
         # This method, `log_metrics`, isn't executed on every iteration but just in case we delay plotting weights.
@@ -304,7 +319,7 @@ class RainbowAgent(AgentBase):
     @staticmethod
     def from_state(state: AgentState) -> AgentBase:
         config = copy.copy(state.config)
-        config.update({'obs_space': state.obs_space, 'action_space': state.action_space})
+        config.update({"obs_space": state.obs_space, "action_space": state.action_space})
         agent = RainbowAgent(**config)
         if state.network is not None:
             agent.set_network(state.network)
@@ -313,8 +328,8 @@ class RainbowAgent(AgentBase):
         return agent
 
     def set_network(self, network_state: NetworkState) -> None:
-        self.net.load_state_dict(network_state.net['net'])
-        self.target_net.load_state_dict(network_state.net['target_net'])
+        self.net.load_state_dict(network_state.net["net"])
+        self.target_net.load_state_dict(network_state.net["target_net"])
 
     def set_buffer(self, buffer_state: BufferState) -> None:
         self.buffer = BufferFactory.from_state(buffer_state)
@@ -337,11 +352,11 @@ class RainbowAgent(AgentBase):
 
         """
         agent_state = torch.load(path)
-        self._config = agent_state.get('config', {})
+        self._config = agent_state.get("config", {})
         self.__dict__.update(**self._config)
 
-        self.net.load_state_dict(agent_state['net'])
-        self.target_net.load_state_dict(agent_state['target_net'])
+        self.net.load_state_dict(agent_state["net"])
+        self.target_net.load_state_dict(agent_state["target_net"])
 
     def save_buffer(self, path: str) -> None:
         """Saves data from the buffer into a file under provided path.
@@ -351,8 +366,9 @@ class RainbowAgent(AgentBase):
 
         """
         import json
+
         dump = self.buffer.dump_buffer(serialize=True)
-        with open(path, 'w') as f:
+        with open(path, "w") as f:
             json.dump(dump, f)
 
     def load_buffer(self, path: str) -> None:
@@ -363,13 +379,16 @@ class RainbowAgent(AgentBase):
 
         """
         import json
-        with open(path, 'r') as f:
+
+        with open(path, "r") as f:
             buffer_dump = json.load(f)
         self.buffer.load_buffer(buffer_dump)
 
     def __eq__(self, o: object) -> bool:
-        return super().__eq__(o) \
-            and isinstance(o, type(self)) \
-            and self._config == o._config \
-            and self.buffer == o.buffer \
+        return (
+            super().__eq__(o)
+            and isinstance(o, type(self))
+            and self._config == o._config
+            and self.buffer == o.buffer
             and self.get_network_state() == o.get_network_state()
+        )
