@@ -1,10 +1,8 @@
-import math
 from functools import lru_cache
 from typing import Optional, Tuple
 
 import torch
 import torch.nn as nn
-import torch.nn.functional as F
 from torch.distributions import Beta, Dirichlet, MultivariateNormal, Normal
 from torch.distributions.distribution import Distribution
 
@@ -21,21 +19,22 @@ class MultivariateGaussianPolicySimple(PolicyType):
     """
     Multivariate Gaussian (Normal) Policy.
 
-    Simplicity of this class, compared to `MultivariateGaussianPolicy`, is due to
-    the assumption that the covariance is sample independent and it is a trainable
-    parameter.
+    Simplicity of this class, compared to `MultivariateGaussianPolicy`, is in
+    the assumption that the covariance is diagonal, sample independent and
+    is treated a trainable parameter.
     """
 
     param_dim = 1
 
-    def __init__(self, size: int, std_init: float=1.0, std_min: float=0.1, std_max: float=3., device=None, **kwargs):
+    def __init__(
+        self, size: int, std_init: float = 0.5, std_min: float = 0.0001, std_max: float = 2.0, device=None, **kwargs
+    ):
         """
         Parameters:
-            size: Size of the observation.
-            batch_size: Expected size of batch. Helps in memory assignments.
-            std_init: (default 2) Initial value for covariance's diagonal. All values start the same.
-            std_min: Minimum value for standard deviation.
-            std_max: Maximum value for standard deviation.
+            size (int): Size of the observation.
+            std_init: Initial value for covariance's diagonal. All values start the same. Default: 0.5.
+            std_min: Minimum value for standard deviation. Default: 0.0001.
+            std_max: Maximum value for standard deviation. Default: 2.
             device: Device where to allocate memory. CPU or CUDA.
 
         """
@@ -57,29 +56,40 @@ class MultivariateGaussianPolicySimple(PolicyType):
     def diag_idx(batch_size: int, size: int, device):
         return torch.arange(size, device=device).repeat((batch_size, 1, 1))
 
-    def forward(self, x) -> Distribution:
-        """Returns distribution"""
-        self.std.data = torch.clamp(self.std, self.std_min, self.std_max)
+    def forward(self, x, deterministic: bool = False) -> Distribution:
+        """Samples from distribution.
+
+        Parameters:
+            x (tensor): Uses a location (mu) for the distrubition.
+            deterministic (bool): Whether to sample from distribution, or use estimates.
+                Default: False, i.e. it'll sample distribution.
+
+        """
+        if deterministic:
+            return x.view(-1, 1)
+
+        x = x.float()  # Need to convert as dist doesn't work on int
+        self.std.data.clamp_(self.std_min, self.std_max)
+
         if self.size == 1:
             return self.dist(x.view(-1, 1), scale=self.std.view(-1, 1))
 
-        # Distinction here is primarily performance optimization (though it isn't too optimal)
         batch_size = x.shape[0]
-        if len(x.shape) == 1 or x.shape[0] == 1:
-            new_shape = (1, self.size, 1)
-            idx = torch.arange(self.size, device=x.device).view(new_shape)
-            std = self._empty_std(batch_size, self.size, x.device).scatter(-1, idx, self.std.repeat(new_shape))
-        else:
-            std = self.std.repeat((batch_size, 1, 1))
-            std = self._empty_std(batch_size, self.size, x.device).scatter(1, self.diag_idx(batch_size, self.size, x.device), std)
-        return self.dist(x, scale_tril=std)
+        new_shape = (batch_size, 1, 1)
+        idx = self.diag_idx(batch_size, self.size, device=x.device)
+        std_shell = self._empty_std(batch_size, self.size, x.device)
+        std_vals = self.std.repeat(new_shape)
+        std = std_shell.scatter(1, idx, std_vals)
+        self._last_samples = {"mu": x, "std": self.std}
+        self._last_dist = self.dist(x, scale_tril=std)
+        return self._last_dist.rsample()
 
     def act(self, x):
-        return x
+        return self.forward(x, deterministic=True)
 
-    @staticmethod
-    def log_prob(dist, samples) -> torch.Tensor:
-        return dist.log_prob(samples)
+    def log_prob(self, samples) -> torch.Tensor:
+        assert self._last_dist is not None, "Need to execute `forward` on data first"
+        return self._last_dist.log_prob(samples)
 
 
 class MultivariateGaussianPolicy(PolicyType):
@@ -95,7 +105,9 @@ class MultivariateGaussianPolicy(PolicyType):
 
     param_dim = 2
 
-    def __init__(self, size: int, std_init: float=1.0, std_min: float=0.1, std_max: float=3., device=None):
+    def __init__(
+        self, size: int, std_init: float = 1.0, std_min: float = 0.001, std_max: float = 2.0, device=None, **kwargs
+    ):
         """
         Parameters:
             size: Observation's dimensionality upon sampling.
@@ -108,6 +120,7 @@ class MultivariateGaussianPolicy(PolicyType):
         self.std_init = std_init
         self.std_min = std_min
         self.std_max = std_max
+        self._last_samples = None
 
     @staticmethod
     @lru_cache(maxsize=10)
@@ -119,29 +132,38 @@ class MultivariateGaussianPolicy(PolicyType):
     def diag_idx(batch_size: int, size: int, device):
         return torch.arange(size, device=device).repeat((batch_size, 1, 1)).view(batch_size, size, 1)
 
-    def forward(self, x) -> Distribution:
+    def forward(self, x, deterministic=False) -> Distribution:
         """Returns distribution"""
         x = x.view(-1, self.param_dim, self.size)
         mu = x[:, 0]
+        if deterministic:
+            return mu.view(-1, 1)
+
         std = torch.clamp(x[:, 1], self.std_min, self.std_max).unsqueeze(-1)
+
         if self.size == 1:
-            return self.dist(mu.view(-1, 1), scale=std.view(-1, 1))
+            _mu = mu.view(-1, 1)
+            _scale = std.view(-1, 1)
+            self._last_samples = {"mu": _mu, "std": torch.diagonal(_scale)}
+            self._last_dist = self.dist(mu.view(-1, 1), scale=_scale)
+            return self._last_dist.rsample()
 
         batch_size = x.shape[0]
-        if x.shape[0] == 1:
-            idx = torch.arange(self.size, device=x.device).view(1, self.size, 1)
-            std = torch.zeros((1, self.size, self.size), device=x.device).scatter(-1, idx, std)
-        else:
-            std = self._empty_std(batch_size, self.size, x.device).scatter(-1, self.diag_idx(batch_size, self.size, x.device), std)
-        return self.dist(mu, scale_tril=std)
+        std_shell = self._empty_std(batch_size, self.size, x.device)
+        idx = self.diag_idx(batch_size, self.size, x.device)
+        std = std_shell.scatter(-1, idx, std)
+        _std_values = torch.diagonal(std.squeeze(0))
+        self._last_samples = {"mu": mu.squeeze(0), "std": _std_values}
+        self._last_dist = self.dist(mu.squeeze(0), std.squeeze(0))
+
+        return self._last_dist.rsample()
 
     def act(self, x) -> torch.Tensor:
         """Deterministic pass. Ignores covariance and returns locations directly."""
         return x.view(-1, self.size, self.param_dim)[..., 0]
 
-    @staticmethod
-    def log_prob(dist, samples) -> torch.Tensor:
-        return dist.log_prob(samples)
+    def log_prob(self, samples):
+        return self._last_dist.log_prob(samples)
 
 
 class GaussianPolicy(PolicyType):
@@ -150,7 +172,7 @@ class GaussianPolicy(PolicyType):
     Has two heads; one for location estimate and one for standard deviation.
     """
 
-    def __init__(self, in_features: FeatureType, out_features: FeatureType, out_scale: float=1, **kwargs):
+    def __init__(self, in_features: FeatureType, out_features: FeatureType, out_scale: float = 1, **kwargs):
         """
         Parameters:
             size: Observation's dimensionality upon sampling.
@@ -171,38 +193,27 @@ class GaussianPolicy(PolicyType):
         self.log_std_max = 2
 
         self._last_dist: Optional[Distribution] = None
-        self._last_samples: Optional[torch.Tensor] = None
+        self._last_samples = None
 
-    @property
-    def logprob(self) -> Optional[torch.Tensor]:
-        if self._last_dist is None or self._last_samples is None:
+    def log_prob(self, samples) -> Optional[torch.Tensor]:
+        if self._last_dist is None:
             return None
+        return self._last_dist.log_prob(samples).sum(axis=-1)
 
-        # *Note*: The note below is borrowed from the SpinningUp implementation.
-        #         Please return once not needed.
-        # Compute logprob from Gaussian, and then apply correction for Tanh squashing.
-        # NOTE: The correction formula is a little bit magic. To get an understanding
-        # of where it comes from, check out the original SAC paper (arXiv 1801.01290)
-        # and look in appendix C. This is a more numerically-stable equivalent to Eq 21.
-        # Try deriving it yourself as a (very difficult) exercise. :)
-        actions = self._last_samples
-        logprob = self._last_dist.log_prob(actions).sum(axis=-1)
-        logprob -= 2*(math.log(2) - actions - F.softplus(-2*actions)).sum(axis=1)
-        return logprob.view(-1, 1)
-
-    def forward(self, x, deterministic=False) -> torch.Tensor:
+    def forward(self, x, deterministic: bool = False) -> torch.Tensor:
         mu = self.mu(x)
-        log_std = self.log_std(x)
-        log_std = torch.clamp(log_std, self.log_std_min, self.log_std_max)
-        std = torch.exp(log_std)
-
         if deterministic:
             self._last_dist, self._last_samples = (None, None)
             return mu
 
+        log_std = self.log_std(x)
+        log_std = torch.clamp(log_std, self.log_std_min, self.log_std_max)
+        std = torch.exp(log_std)
+
         self._last_dist = dist = self.dist(mu, std)
-        self._last_samples = actions = dist.rsample()
-        return self.out_scale * torch.tanh(actions)
+        actions = dist.rsample()
+        self._last_samples = {"mu": mu.squeeze(0), "std": std}
+        return actions
 
 
 class BetaPolicy(PolicyType):
@@ -218,7 +229,7 @@ class BetaPolicy(PolicyType):
 
     param_dim = 2
 
-    def __init__(self, size: int, bounds: Tuple[float, float]=(1, float('inf'))):
+    def __init__(self, size: int, bounds: Tuple[float, float] = (1, float("inf"))):
         """
         Parameters:
             size: Observation's dimensionality upon sampling.
@@ -246,7 +257,7 @@ class DirichletPolicy(PolicyType):
 
     param_dim = 1
 
-    def __init__(self, *, alpha_min: float=0.05):
+    def __init__(self, *, alpha_min: float = 0.05):
         super(DirichletPolicy, self).__init__()
         self.alpha_min = alpha_min
 
