@@ -3,14 +3,14 @@ import pylab as plt
 import torch
 
 from ai_traineree.agents.dqn import DQNAgent
-from ai_traineree.loggers import TensorboardLogger
+from ai_traineree.loggers.file_logger import FileLogger
 from ai_traineree.runners.env_runner import EnvRunner
 from ai_traineree.tasks import GymTask
 
 seed = 32167
 # torch.set_deterministic(True)
 torch.manual_seed(seed)
-data_logger = TensorboardLogger()
+data_logger = FileLogger(filepath="cart_dqn")
 
 env_name = "CartPole-v1"
 task = GymTask(env_name, seed=seed)
