@@ -2,7 +2,7 @@ import numpy as np
 import pylab as plt
 
 from ai_traineree.agents.rainbow import RainbowAgent as Agent
-from ai_traineree.loggers import TensorboardLogger
+from ai_traineree.loggers.tensorboard_logger import TensorboardLogger
 from ai_traineree.runners.env_runner import EnvRunner
 from ai_traineree.tasks import GymTask
 
