@@ -5,7 +5,7 @@ import pylab as plt
 import torch
 
 from ai_traineree.agents.sac import SACAgent
-from ai_traineree.loggers import TensorboardLogger
+from ai_traineree.loggers.tensorboard_logger import TensorboardLogger
 from ai_traineree.runners.env_runner import EnvRunner
 from ai_traineree.tasks import GymTask
 from ai_traineree.types import TaskType
